package com.topcoder.db.generation.xsd2ddl;

import com.topcoder.db.generation.xsd2ddl.data.FieldType;

/**
 * This interface abstracts the type mapping between a definition source and a target.
 * 
 * This interface takes a FieldType object as input and decides in an implementation
 * specific way what would be the best type to represent the source type in the target
 * type system.
 *
 * Essentially what this interface does is converting the field type definition 
 * from an implementation independent format represented by the FieldType class in the data package
 * to an implementation specific format (generally an SQL type for some specific database). 
 * 
 * Note:
 * 
 * All implementations are expected to have a public constructor taking a configuration
 * namespace (String) as parameter. This constructor is invoked by the DefinitionRenderer
 * implementations that need such type mapping service. This allows the TypeMapping implementation
 * itself to read configuration information.
 *
 * Thread safety:
 * 
 * The implementations are typically thread safe because they will most likely
 * not need to update state during type determination. An implementation is likely to handle
 * concurrent render calls in a thread safe manner (but it is not a requirement).
 * 
 * @author TCSDESIGNER
 */
public interface TypeMapping {

	/**
	 * Converts a source field type to a target type. 
	 * The target type is a string suitable in SQL statements for example.
	 *  
	 * The method to do the conversion is specific to each implementation.
	 * 
	 * @param type the field type the mapped type is obtained for
	 * @return the type that can be used in the destination system
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 */
	public String convertType(FieldType type) throws IllegalArgumentException, TypeMappingException;
	
}
